import terrascript.core as core


@core.schema
class ActionThreshold(core.Schema):

    action_threshold_type: str | core.StringOut = core.attr(str)

    action_threshold_value: float | core.FloatOut = core.attr(float)

    def __init__(
        self,
        *,
        action_threshold_type: str | core.StringOut,
        action_threshold_value: float | core.FloatOut,
    ):
        super().__init__(
            args=ActionThreshold.Args(
                action_threshold_type=action_threshold_type,
                action_threshold_value=action_threshold_value,
            ),
        )

    @core.schema_args
    class Args(core.SchemaArgs):
        action_threshold_type: str | core.StringOut = core.arg()

        action_threshold_value: float | core.FloatOut = core.arg()


@core.schema
class Subscriber(core.Schema):

    address: str | core.StringOut = core.attr(str)

    subscription_type: str | core.StringOut = core.attr(str)

    def __init__(
        self,
        *,
        address: str | core.StringOut,
        subscription_type: str | core.StringOut,
    ):
        super().__init__(
            args=Subscriber.Args(
                address=address,
                subscription_type=subscription_type,
            ),
        )

    @core.schema_args
    class Args(core.SchemaArgs):
        address: str | core.StringOut = core.arg()

        subscription_type: str | core.StringOut = core.arg()


@core.schema
class IamActionDefinition(core.Schema):

    groups: list[str] | core.ArrayOut[core.StringOut] | None = core.attr(
        str, default=None, kind=core.Kind.array
    )

    policy_arn: str | core.StringOut = core.attr(str)

    roles: list[str] | core.ArrayOut[core.StringOut] | None = core.attr(
        str, default=None, kind=core.Kind.array
    )

    users: list[str] | core.ArrayOut[core.StringOut] | None = core.attr(
        str, default=None, kind=core.Kind.array
    )

    def __init__(
        self,
        *,
        policy_arn: str | core.StringOut,
        groups: list[str] | core.ArrayOut[core.StringOut] | None = None,
        roles: list[str] | core.ArrayOut[core.StringOut] | None = None,
        users: list[str] | core.ArrayOut[core.StringOut] | None = None,
    ):
        super().__init__(
            args=IamActionDefinition.Args(
                policy_arn=policy_arn,
                groups=groups,
                roles=roles,
                users=users,
            ),
        )

    @core.schema_args
    class Args(core.SchemaArgs):
        groups: list[str] | core.ArrayOut[core.StringOut] | None = core.arg(default=None)

        policy_arn: str | core.StringOut = core.arg()

        roles: list[str] | core.ArrayOut[core.StringOut] | None = core.arg(default=None)

        users: list[str] | core.ArrayOut[core.StringOut] | None = core.arg(default=None)


@core.schema
class SsmActionDefinition(core.Schema):

    action_sub_type: str | core.StringOut = core.attr(str)

    instance_ids: list[str] | core.ArrayOut[core.StringOut] = core.attr(str, kind=core.Kind.array)

    region: str | core.StringOut = core.attr(str)

    def __init__(
        self,
        *,
        action_sub_type: str | core.StringOut,
        instance_ids: list[str] | core.ArrayOut[core.StringOut],
        region: str | core.StringOut,
    ):
        super().__init__(
            args=SsmActionDefinition.Args(
                action_sub_type=action_sub_type,
                instance_ids=instance_ids,
                region=region,
            ),
        )

    @core.schema_args
    class Args(core.SchemaArgs):
        action_sub_type: str | core.StringOut = core.arg()

        instance_ids: list[str] | core.ArrayOut[core.StringOut] = core.arg()

        region: str | core.StringOut = core.arg()


@core.schema
class ScpActionDefinition(core.Schema):

    policy_id: str | core.StringOut = core.attr(str)

    target_ids: list[str] | core.ArrayOut[core.StringOut] = core.attr(str, kind=core.Kind.array)

    def __init__(
        self,
        *,
        policy_id: str | core.StringOut,
        target_ids: list[str] | core.ArrayOut[core.StringOut],
    ):
        super().__init__(
            args=ScpActionDefinition.Args(
                policy_id=policy_id,
                target_ids=target_ids,
            ),
        )

    @core.schema_args
    class Args(core.SchemaArgs):
        policy_id: str | core.StringOut = core.arg()

        target_ids: list[str] | core.ArrayOut[core.StringOut] = core.arg()


@core.schema
class Definition(core.Schema):

    iam_action_definition: IamActionDefinition | None = core.attr(IamActionDefinition, default=None)

    scp_action_definition: ScpActionDefinition | None = core.attr(ScpActionDefinition, default=None)

    ssm_action_definition: SsmActionDefinition | None = core.attr(SsmActionDefinition, default=None)

    def __init__(
        self,
        *,
        iam_action_definition: IamActionDefinition | None = None,
        scp_action_definition: ScpActionDefinition | None = None,
        ssm_action_definition: SsmActionDefinition | None = None,
    ):
        super().__init__(
            args=Definition.Args(
                iam_action_definition=iam_action_definition,
                scp_action_definition=scp_action_definition,
                ssm_action_definition=ssm_action_definition,
            ),
        )

    @core.schema_args
    class Args(core.SchemaArgs):
        iam_action_definition: IamActionDefinition | None = core.arg(default=None)

        scp_action_definition: ScpActionDefinition | None = core.arg(default=None)

        ssm_action_definition: SsmActionDefinition | None = core.arg(default=None)


@core.resource(type="aws_budgets_budget_action", namespace="budgets")
class BudgetAction(core.Resource):

    account_id: str | core.StringOut | None = core.attr(str, default=None, computed=True)

    action_id: str | core.StringOut = core.attr(str, computed=True)

    action_threshold: ActionThreshold = core.attr(ActionThreshold)

    action_type: str | core.StringOut = core.attr(str)

    approval_model: str | core.StringOut = core.attr(str)

    arn: str | core.StringOut = core.attr(str, computed=True)

    budget_name: str | core.StringOut = core.attr(str)

    definition: Definition = core.attr(Definition)

    execution_role_arn: str | core.StringOut = core.attr(str)

    id: str | core.StringOut = core.attr(str, computed=True)

    notification_type: str | core.StringOut = core.attr(str)

    status: str | core.StringOut = core.attr(str, computed=True)

    subscriber: list[Subscriber] | core.ArrayOut[Subscriber] = core.attr(
        Subscriber, kind=core.Kind.array
    )

    def __init__(
        self,
        resource_name: str,
        *,
        action_threshold: ActionThreshold,
        action_type: str | core.StringOut,
        approval_model: str | core.StringOut,
        budget_name: str | core.StringOut,
        definition: Definition,
        execution_role_arn: str | core.StringOut,
        notification_type: str | core.StringOut,
        subscriber: list[Subscriber] | core.ArrayOut[Subscriber],
        account_id: str | core.StringOut | None = None,
        depends_on: list[str] | core.ArrayOut[core.StringOut] | None = None,
        provider: str | core.StringOut | None = None,
        lifecycle: core.Lifecycle | None = None,
    ):
        super().__init__(
            name=resource_name,
            args=BudgetAction.Args(
                action_threshold=action_threshold,
                action_type=action_type,
                approval_model=approval_model,
                budget_name=budget_name,
                definition=definition,
                execution_role_arn=execution_role_arn,
                notification_type=notification_type,
                subscriber=subscriber,
                account_id=account_id,
                depends_on=depends_on,
                provider=provider,
                lifecycle=lifecycle,
            ),
        )

    @core.schema_args
    class Args(core.Resource.Args):
        account_id: str | core.StringOut | None = core.arg(default=None)

        action_threshold: ActionThreshold = core.arg()

        action_type: str | core.StringOut = core.arg()

        approval_model: str | core.StringOut = core.arg()

        budget_name: str | core.StringOut = core.arg()

        definition: Definition = core.arg()

        execution_role_arn: str | core.StringOut = core.arg()

        notification_type: str | core.StringOut = core.arg()

        subscriber: list[Subscriber] | core.ArrayOut[Subscriber] = core.arg()
