import terrascript.core as core


@core.resource(type="aws_glue_schema", namespace="glue")
class Schema(core.Resource):

    arn: str | core.StringOut = core.attr(str, computed=True)

    compatibility: str | core.StringOut = core.attr(str)

    data_format: str | core.StringOut = core.attr(str)

    description: str | core.StringOut | None = core.attr(str, default=None)

    id: str | core.StringOut = core.attr(str, computed=True)

    latest_schema_version: int | core.IntOut = core.attr(int, computed=True)

    next_schema_version: int | core.IntOut = core.attr(int, computed=True)

    registry_arn: str | core.StringOut | None = core.attr(str, default=None, computed=True)

    registry_name: str | core.StringOut = core.attr(str, computed=True)

    schema_checkpoint: int | core.IntOut = core.attr(int, computed=True)

    schema_definition: str | core.StringOut = core.attr(str)

    schema_name: str | core.StringOut = core.attr(str)

    tags: dict[str, str] | core.MapOut[core.StringOut] | None = core.attr(
        str, default=None, kind=core.Kind.map
    )

    tags_all: dict[str, str] | core.MapOut[core.StringOut] | None = core.attr(
        str, default=None, computed=True, kind=core.Kind.map
    )

    def __init__(
        self,
        resource_name: str,
        *,
        compatibility: str | core.StringOut,
        data_format: str | core.StringOut,
        schema_definition: str | core.StringOut,
        schema_name: str | core.StringOut,
        description: str | core.StringOut | None = None,
        registry_arn: str | core.StringOut | None = None,
        tags: dict[str, str] | core.MapOut[core.StringOut] | None = None,
        tags_all: dict[str, str] | core.MapOut[core.StringOut] | None = None,
        depends_on: list[str] | core.ArrayOut[core.StringOut] | None = None,
        provider: str | core.StringOut | None = None,
        lifecycle: core.Lifecycle | None = None,
    ):
        super().__init__(
            name=resource_name,
            args=Schema.Args(
                compatibility=compatibility,
                data_format=data_format,
                schema_definition=schema_definition,
                schema_name=schema_name,
                description=description,
                registry_arn=registry_arn,
                tags=tags,
                tags_all=tags_all,
                depends_on=depends_on,
                provider=provider,
                lifecycle=lifecycle,
            ),
        )

    @core.schema_args
    class Args(core.Resource.Args):
        compatibility: str | core.StringOut = core.arg()

        data_format: str | core.StringOut = core.arg()

        description: str | core.StringOut | None = core.arg(default=None)

        registry_arn: str | core.StringOut | None = core.arg(default=None)

        schema_definition: str | core.StringOut = core.arg()

        schema_name: str | core.StringOut = core.arg()

        tags: dict[str, str] | core.MapOut[core.StringOut] | None = core.arg(default=None)

        tags_all: dict[str, str] | core.MapOut[core.StringOut] | None = core.arg(default=None)
