import terrascript.core as core


@core.resource(type="aws_shield_protection_group", namespace="shield")
class ProtectionGroup(core.Resource):

    aggregation: str | core.StringOut = core.attr(str)

    id: str | core.StringOut = core.attr(str, computed=True)

    members: list[str] | core.ArrayOut[core.StringOut] | None = core.attr(
        str, default=None, kind=core.Kind.array
    )

    pattern: str | core.StringOut = core.attr(str)

    protection_group_arn: str | core.StringOut = core.attr(str, computed=True)

    protection_group_id: str | core.StringOut = core.attr(str)

    resource_type: str | core.StringOut | None = core.attr(str, default=None)

    tags: dict[str, str] | core.MapOut[core.StringOut] | None = core.attr(
        str, default=None, kind=core.Kind.map
    )

    tags_all: dict[str, str] | core.MapOut[core.StringOut] | None = core.attr(
        str, default=None, computed=True, kind=core.Kind.map
    )

    def __init__(
        self,
        resource_name: str,
        *,
        aggregation: str | core.StringOut,
        pattern: str | core.StringOut,
        protection_group_id: str | core.StringOut,
        members: list[str] | core.ArrayOut[core.StringOut] | None = None,
        resource_type: str | core.StringOut | None = None,
        tags: dict[str, str] | core.MapOut[core.StringOut] | None = None,
        tags_all: dict[str, str] | core.MapOut[core.StringOut] | None = None,
        depends_on: list[str] | core.ArrayOut[core.StringOut] | None = None,
        provider: str | core.StringOut | None = None,
        lifecycle: core.Lifecycle | None = None,
    ):
        super().__init__(
            name=resource_name,
            args=ProtectionGroup.Args(
                aggregation=aggregation,
                pattern=pattern,
                protection_group_id=protection_group_id,
                members=members,
                resource_type=resource_type,
                tags=tags,
                tags_all=tags_all,
                depends_on=depends_on,
                provider=provider,
                lifecycle=lifecycle,
            ),
        )

    @core.schema_args
    class Args(core.Resource.Args):
        aggregation: str | core.StringOut = core.arg()

        members: list[str] | core.ArrayOut[core.StringOut] | None = core.arg(default=None)

        pattern: str | core.StringOut = core.arg()

        protection_group_id: str | core.StringOut = core.arg()

        resource_type: str | core.StringOut | None = core.arg(default=None)

        tags: dict[str, str] | core.MapOut[core.StringOut] | None = core.arg(default=None)

        tags_all: dict[str, str] | core.MapOut[core.StringOut] | None = core.arg(default=None)
